/* This source file is part of the Procept Operating System */
 
/*This file is prepared for Doxygen automatic documentation generation.*/
 
/*! @file *********************************************************************
 *
 * @brief The implementation of the functionalities needed 
 *        for handling the PACK module.
 *
 * - Compiler:           uVision5 for nRF51
 * - Supported devices:  nRF51822xxAA
 * - AppNote:
 *
 * @author               tchinnadurai
 ******************************************************************************/
 
/* Copyright (c) 2011 Procept IP. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted unless the following conditions are met.
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of Procept may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with a Procept
 * Product with permission of the Procept Board of Directors.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCEPT "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 **/

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "nrf_gpio.h"
#include "nrf_drv_gpiote.h"
#include "pack.h"
#include "app_error.h"
#include "uart.h"

pack_buffer_t p_buffer[NUM_PBUFF_ENTRIES];
uint8_t uartBuffer[MAX_DATA_LEN+NUM_OF_USED_BYTES];
static uint16_t index_count_pbuffer=0x0;//0xffff;
static uint16_t pck_writePopulateTxBufferIndex=0x0;
static uint16_t pck_readPopulateTxBufferIndex=0x0;
static uint16_t index_count_uint16 = 0;
static uint16_t crc_16 = 0xffff;
static uint16_t pack_calculateTxCRC(uint16_t index);
static uint32_t pack_debug_1=0;
#define MAX_SIZE_DEBUG_BIG_BUFFER	100
static uint8_t debug_big_buffer[MAX_SIZE_DEBUG_BIG_BUFFER];
static uint32_t big_bff_ctr =0;
static uint16_t pack_calculateTxResponseCRC(uint8_t* buffer);
static uint16_t pack_calculateCRCFromNRFtoSTM(uint8_t* buffer_to_send);
/************************************************
 * 				PUBLIC ROUTINES					*
 ************************************************/
/*!
*	@brief 	Function for the PACK Buffer initialization.
*	@param[in] None. 
*	@return None.
*/
void pack_init(void)
{
	for(uint16_t i=0;i<NUM_PBUFF_ENTRIES;i++)
	{
		memset(&p_buffer[i], 0, sizeof(p_buffer));
	}
	//from this for debugging 
	memset(&debug_big_buffer, 0, sizeof(debug_big_buffer));
	//till this for debugging
}
void error_occurred(void)
{
	__NOP;
}
uint16_t pack_getWriteTxBufferIndex(void)
{
	return pck_writePopulateTxBufferIndex;
}
void pack_resetWriteTxBufferIndex(void)
{
	pck_writePopulateTxBufferIndex=0;
}
void pack_incrementWriteTxBufferIndex(void)
{
	pck_writePopulateTxBufferIndex++;
}
uint16_t pack_getReadTxBufferIndex(void)
{
	return pck_readPopulateTxBufferIndex;
}
void pack_setReadTxBufferIndex(uint16_t index)
{
	pck_readPopulateTxBufferIndex = index;
}
void pack_resetReadTxBufferIndex(void)
{
	pck_readPopulateTxBufferIndex=0;
}
void pack_incrementReadTxBufferIndex(void)
{
		if(pck_readPopulateTxBufferIndex==(NUM_PBUFF_ENTRIES-1))
		{
			pack_resetReadTxBufferIndex();				
		}
		else
		{
			pck_readPopulateTxBufferIndex++;
		}	
}
uint16_t pack_findArrayIndex(uint16_t index)
{
	uint16_t error_code = OUT_OF_RANGE;
	for(uint16_t i=0;i<NUM_PBUFF_ENTRIES;i++)
	{
		if(p_buffer[i].index==index)
		{
			return i;
		}
	}
	return error_code;
}
uint16_t pack_swapLsbAndMsb(uint16_t index)
{
	uint16_t returnValue=index>>8;
	returnValue|=index<<8;
}
void pack_clearBuffer(uint16_t index)
{
	uint16_t swappedIndex=pack_swapLsbAndMsb(index);
	uint16_t arrayIndex=pack_findArrayIndex(index);

	if(arrayIndex!=OUT_OF_RANGE)
	{
		p_buffer[arrayIndex].sender=0x0;			
		p_buffer[arrayIndex].index=0x0;
		p_buffer[arrayIndex].id=0x0;
		p_buffer[arrayIndex].inUse=0x0;
		p_buffer[arrayIndex].crc=0x0;
		p_buffer[arrayIndex].isPacked=0x0;
		for(uint8_t i=0;i<MAX_DATA_LEN;i++)
		{
			p_buffer[arrayIndex].data[i]=0x0;
		}	
		p_buffer[arrayIndex].isSent=0x0;//todo make this 0 only when ACK is received back
		p_buffer[arrayIndex].retryCount=0x0;//todo increment every time retry happens (bec of NACK/time out)
		p_buffer[arrayIndex].timerIntrptCount=0x0;
	}
	else
	{
		error_occurred();
	}
}
/*!
*	@brief 	The code in this module populates the transmit buffer
*					with command ID received from remote.
*					
* @detail                                       
*					 Since this module is called in nus_data_handler(similar to interrupt
*					 service routine), this needs to be executed as quick as possible.
*					 Hence the concept of NO_EMPTY_ENTRIES and non_empty_entries have been used.
*	@param[in] input: The command ID received from remote.
*	@return NO_EMPTY_ENTRIES: Buffer is full
*					SUCCESS					: Successfully entered the command ID.
*/
uint16_t pack_writePopulateTxBuffer_old(uint8_t input)
{
	uint8_t non_empty_entries=0;
	findEmptyEntry:
	if(index_count_pbuffer==(NUM_PBUFF_ENTRIES-1))
	{
		index_count_pbuffer=0;
	}
	else
	{
		index_count_pbuffer++;
	}

	if(p_buffer[index_count_pbuffer].inUse == 0)
	{
		p_buffer[index_count_pbuffer].id = input;
		p_buffer[index_count_pbuffer].inUse = 1;
		p_buffer[index_count_pbuffer].index = index_count_uint16++;//index would roll over to 0 after 65535 entries.
		pck_writePopulateTxBufferIndex = index_count_pbuffer;
	}
	else
	{
		non_empty_entries++;
		if(non_empty_entries<NUM_PBUFF_ENTRIES)
		{
			goto findEmptyEntry;
		}
		else
		{//All the entries in the p_buffer buffer are in use (meaning: yet
			//to be transmitted via uart to STM32)
			return NO_EMPTY_ENTRIES;
		}
	}
	return SUCCESS;
}

uint16_t pack_writePopulateTxBuffer(uint8_t input)
{
	uint8_t non_empty_entries=0;

	if(p_buffer[index_count_pbuffer].inUse == 0)
	{
		p_buffer[index_count_pbuffer].id = input;
		p_buffer[index_count_pbuffer].inUse = 1;
		p_buffer[index_count_pbuffer].index = index_count_uint16++;//index would roll over to 0 after 65535 entries.
		p_buffer[index_count_pbuffer].timerIntrptCount=0;//timer_getRtc1Count();
		//pck_writePopulateTxBufferIndex = index_count_pbuffer;
		if(index_count_pbuffer==(NUM_PBUFF_ENTRIES-1))
		{
			index_count_pbuffer=0;
			pack_resetWriteTxBufferIndex();				
		}
		else
		{
			index_count_pbuffer++;
			pack_incrementWriteTxBufferIndex();
		}
	}
	else
	{
		findEmptyEntry:
		non_empty_entries++;
		if(index_count_pbuffer==(NUM_PBUFF_ENTRIES-1))
		{
			index_count_pbuffer=0;
			if(p_buffer[index_count_pbuffer].inUse == 0)
			{
				pack_resetWriteTxBufferIndex();				
			}						
		}
		else
		{
			index_count_pbuffer++;
			if(p_buffer[index_count_pbuffer].inUse == 0)
			{
				pack_incrementWriteTxBufferIndex();				
			}			
		}		
		if(p_buffer[index_count_pbuffer].inUse == 0)
		{
			p_buffer[index_count_pbuffer].id = input;
			p_buffer[index_count_pbuffer].inUse = 1;
			p_buffer[index_count_pbuffer].index = index_count_uint16++;//index would roll over to 0 after 65535 entries.
			p_buffer[index_count_pbuffer].timerIntrptCount=0;//timer_getRtc1Count();
			//pck_writePopulateTxBufferIndex = index_count_pbuffer;
		}
		else
		{		
			if(non_empty_entries<NUM_PBUFF_ENTRIES)
			{
				goto findEmptyEntry;
			}
			else
			{//All the entries in the p_buffer buffer are in use (meaning: yet
				//to be transmitted via uart to STM32)
				return NO_EMPTY_ENTRIES;
			}
		}
	}
	return SUCCESS;
}

void pack_checkIfTimeoutHappened(void)
{
	//scan through all the entries that have inUse=1
	//and check if timeout has occurred since the 
	//packet was first received in nus_data_handler
	uint32_t currentTmrIntrpt=0;//timer_getRtc1Count();
	for(uint32_t i=0;i<NUM_PBUFF_ENTRIES;i++)
	{
		if(p_buffer[i].inUse==0x01)
		{
			uint32_t timerDifference=0;
			if(p_buffer[i].timerIntrptCount>currentTmrIntrpt)
			{
				timerDifference=p_buffer[i].timerIntrptCount-currentTmrIntrpt;
			}
			else
			{
				timerDifference=currentTmrIntrpt-p_buffer[i].timerIntrptCount;
			}
			if(timerDifference>UART_TX_TIMEOUT)
			{
				//discard this tx packet. 
				//todo: Later copy this to flash memory for further debugging
				p_buffer[i].sender=0x0;					
				p_buffer[i].index=0x0;
				p_buffer[i].id=0x0;
				p_buffer[i].inUse=0x0;
				p_buffer[i].crc=0x0;
				p_buffer[i].isPacked=0x0;
				for(uint8_t i=0;i<MAX_DATA_LEN;i++)
				{
					p_buffer[i].data[i]=0x0;
				}	
				p_buffer[i].isSent=0x0;//todo make this 0 only when ACK is received back
				p_buffer[i].retryCount=0x0;//todo increment every time retry happens (bec of NACK/time out)
				p_buffer[i].timerIntrptCount=0x0;				
			}
		}
	}
}

void pack_readPopulateTxBuffer(uint16_t index)
{
	if((p_buffer[index].isPacked==0) && (p_buffer[index].inUse!=0))
	{
		//This entry has not been packed yet, so, pack it now.
		p_buffer[index].isPacked = 1;
		p_buffer[index].sender = SENDER_NRF_COMMAND;		
		for(uint8_t i=0;i<MAX_DATA_LEN;i++)
		{
			p_buffer[index].data[i]=0x0;
		}
		p_buffer[index].crc=pack_calculateTxCRC(index);
	}
}

void pack_packInBufferAndSendViaUart(uint16_t index)
{
	uint8_t uartBufCnt=0;
	memset(&uartBuffer, 0, sizeof(uartBuffer));
	if((p_buffer[index].isPacked!=0) && (p_buffer[index].inUse!=0))
	{	
		uartBuffer[uartBufCnt++]=SENDER_NRF_COMMAND;//sender			
		uartBuffer[uartBufCnt++]=(p_buffer[index].index)>>8;//Index -LSB
		uartBuffer[uartBufCnt++]=p_buffer[index].index;//Index -MSB	
		uartBuffer[uartBufCnt++]=p_buffer[index].id;//cmd id
		for(uint8_t i=0;i<MAX_DATA_LEN;i++)
		{	
			uartBuffer[uartBufCnt++]=p_buffer[index].data[i];
		}

		uartBuffer[uartBufCnt++]=(p_buffer[index].crc)>>8;//CRC -LSB
		uartBuffer[uartBufCnt++]=p_buffer[index].crc;//CRC -MSB
		for(uint8_t i=0;i<uartBufCnt;i++)
		{
			//uart0Tx_Byte(uartBuffer[i]);
			while(app_uart_put(uartBuffer[i]) != NRF_SUCCESS);
			//while(app_uart_put(0x80+i) != NRF_SUCCESS);
			//__NOP();
			app_uart_flush();			
		}
//from this for debugging
		/*for(uint8_t i=0;i<uartBufCnt;i++)
		{
			debug_big_buffer[big_bff_ctr++]=uartBuffer[i];
			if(big_bff_ctr==MAX_SIZE_DEBUG_BIG_BUFFER)
			{
				big_bff_ctr=0;
			}
		}*/
		
//till this for debugging		
		__NOP;
	}
}

void pack_sendResponseToSTM32(uint8_t * buffer, uint8_t ack_or_nack)
{
	uint8_t uartBufCnt=0;
	memset(&uartBuffer, 0, sizeof(uartBuffer));	
	uartBuffer[uartBufCnt++]=SENDER_NRF_RESPONSE;//sender
	*buffer++;//ignore first because it is the sender id
	uartBuffer[uartBufCnt++]=*buffer++;//index
	uartBuffer[uartBufCnt++]=*buffer++;//index
	uartBuffer[uartBufCnt++]=ack_or_nack;//ack/nack	
	uint16_t calculated_crc = pack_calculateTxResponseCRC(&uartBuffer[0]);
	uartBuffer[uartBufCnt++]=calculated_crc>>8;
	uartBuffer[uartBufCnt++]=calculated_crc;
	for(uint8_t i=0;i<uartBufCnt;i++)
	{
		while(app_uart_put(uartBuffer[i]) != NRF_SUCCESS);
		app_uart_flush();			
	}	
	//nrf_delay_ms(1);
	if(uart_getTxCtr()==0x6)
	{
		uart_clearTxCtr();
		//led_turnGreenToggle();
	}
	else
	{
		uart_clearTxCtr();
		//led_turnRedOn();
	}
}
uint16_t pack_incrementRetryCountAndResend(uint16_t index)
{
	uint16_t swappedIndex=pack_swapLsbAndMsb(index);	
	uint8_t uartBufCnt=0;
	memset(&uartBuffer, 0, sizeof(uartBuffer));

	if((p_buffer[swappedIndex].isPacked!=0) && (p_buffer[swappedIndex].inUse!=0) && (p_buffer[swappedIndex].retryCount<MAX_RETRY_COUNT))
	{	
		uartBuffer[uartBufCnt++]=(p_buffer[swappedIndex].index)>>8;//Index -LSB
		uartBuffer[uartBufCnt++]=p_buffer[swappedIndex].index;//Index -MSB	
		uartBuffer[uartBufCnt++]=p_buffer[swappedIndex].id;//cmd id
		for(uint8_t i=0;i<MAX_DATA_LEN;i++)
		{	
			uartBuffer[uartBufCnt++]=p_buffer[swappedIndex].data[i];
		}

		uartBuffer[uartBufCnt++]=(p_buffer[swappedIndex].crc)>>8;//CRC -LSB
		uartBuffer[uartBufCnt++]=p_buffer[swappedIndex].crc;//CRC -MSB
		for(uint8_t i=0;i<uartBufCnt;i++)
		{
			//uart0Tx_Byte(uartBuffer[i]);
			while(app_uart_put(uartBuffer[i]) != NRF_SUCCESS);
			//while(app_uart_put(0x80+i) != NRF_SUCCESS);
			//__NOP();
			app_uart_flush();			
		}
		p_buffer[swappedIndex].retryCount++;	
		p_buffer[swappedIndex].timerIntrptCount=0;//timer_getRtc1Count();
		return SUCCESS;
	}
	else
	{
		return OUT_OF_RANGE;
	}
	
}
/*!
*	@brief 	The code in this module generates the crc for a block of data.
*					This does CRC check byte by byte.
*					Reference: http://my.ece.msstate.edu/faculty/reese/EE4743/crcexplained.htm
* @detail                                       16   12   5
*					 this is the CCITT CRC 16 polynomial X  + X  + X  + 1.
*					 This works out to be 0x1021, but the way the algorithm works
*					 lets us use 0x8408 (the reverse of the bit pattern).  The high
*					 bit is always assumed to be set, thus we only use 16 bits to
*					 represent the 17 bit value.
*	@param[in] data_p: The value for which the CRC needs to be calculated.
*	@return None.
*/
static void crc16Full(uint8_t data_p)
{
      uint8_t i;
			for (i=0; i < 8;i++)
			{
						if ((crc_16 & 0x0001) ^ ( data_p & 0x0001))
									crc_16 = (crc_16 >> 1) ^ POLY;
						else  {crc_16 >>= 1;}
						data_p >>= 1;
			}

}

/*!
*	@brief 	Function for clearing the global variable for CRC
*	@param[in] None
*	@return None.
*/
static void clear_crc_16(void)
{
	crc_16 = 0xffff;
}

/*!
*	@brief 	Function for returning the global variable for CRC
*	@param[in] None
*	@return crc_16: Calculated CRC
*/
static uint16_t getCrc16(void)
{
	return crc_16;
}

/*!
*	@brief 	Function to calculate CRC for the data that is to be sent out of nRF51
*	@param[in] 		None
* @retval     	calculated CRC
*/
static uint16_t pack_calculateTxCRC(uint16_t index)
{
	clear_crc_16();
	uint16_t Index=p_buffer[index].index;
	crc16Full(p_buffer[index].sender);		
	crc16Full((Index)>>8);
	crc16Full(Index);	
	crc16Full(p_buffer[index].id);
	for(uint8_t i=0;i<MAX_DATA_LEN;i++)
	{	
		crc16Full(p_buffer[index].data[i]);
	}
	return (getCrc16());
}

uint16_t pack_calculateRxCRC(uint8_t *buffer)
{
	clear_crc_16();
	crc16Full(*buffer++);	//sender	
	crc16Full(*buffer++);	//index
	crc16Full(*buffer++);	//index	
	crc16Full(*buffer++);	//id
	for(uint8_t i=0;i<MAX_DATA_LEN;i++)
	{	
		crc16Full(*buffer++);	//data
	}
	uint16_t calculatedCRC=crc_16;
	uint16_t receivedCRC=(*buffer++)<<8;
	receivedCRC|=*buffer++;
	if(calculatedCRC!=receivedCRC)
	{
		return CRC_MISMATCH;
	}
	else
	{
		return SUCCESS;
	}	
}

static uint16_t pack_calculateTxResponseCRC(uint8_t* buffer)
{
	clear_crc_16();
	crc16Full(*buffer++);		//sender
	crc16Full(*buffer++);		//index	
	crc16Full(*buffer++);		//index	
	crc16Full(*buffer++);		//ack/nack		
	return (getCrc16());
}
uint16_t pack_findCrcToDecideNumberOfBytesToRead(uint8_t *rxedBuffer, uint32_t number_of_received_bytes_minus_crc)
{
	clear_crc_16();	
	for(uint32_t i=0;i<number_of_received_bytes_minus_crc;i++)
	{
		crc16Full(*rxedBuffer++);
	}
	return (getCrc16());	
}

void pack_sendCommandFromNRFtoSTM(void)
{
	uint8_t uartBufCnt=0;
	memset(&uartBuffer, 0, sizeof(uartBuffer));
	uartBuffer[uartBufCnt++]=SENDER_NRF_COMMAND;//sender
	uartBuffer[uartBufCnt++]=index_count_uint16>>8;//index
	uartBuffer[uartBufCnt++]=index_count_uint16;//index
	uartBuffer[uartBufCnt++]=0x03;//example command ID
	for(uint8_t i=0;i<MAX_DATA_LEN;i++)
	{	
		uartBuffer[uartBufCnt++]=i;//data
	}
	uint16_t calculated_crc=pack_calculateCRCFromNRFtoSTM(&uartBuffer[0]);
	uartBuffer[uartBufCnt++]=calculated_crc>>8;
	uartBuffer[uartBufCnt++]=calculated_crc;
	
	for(uint8_t i=0;i<uartBufCnt;i++)
	{
		while(app_uart_put(uartBuffer[i]) != NRF_SUCCESS);
		app_uart_flush();			
	}
	//nrf_delay_ms(1);
	if(uart_getTxCtr()==0x10)
	{
		uart_clearTxCtr();
		//led_turnGreenToggle();
	}
	else
	{
		uart_clearTxCtr();
		//led_turnRedOn();
	}
}
static uint16_t pack_calculateCRCFromNRFtoSTM(uint8_t* buffer_to_send)
{
	clear_crc_16();
	crc16Full(*buffer_to_send++);//sender	
	crc16Full(*buffer_to_send++);//index
	crc16Full(*buffer_to_send++);	//index
	crc16Full(*buffer_to_send++);//command id

	for(uint8_t i=0;i<MAX_DATA_LEN;i++)
	{	
		crc16Full(*buffer_to_send++);//data
	}
	return (getCrc16());
}
/*!
*	@brief 	Function to calculate CRC for the data that is to be received into nRF51
*	@param[in] 		*rxedBuffer: Pointer containing the data that need to be checked for CRC.
* @retval     	CRC_MISMATCH: CRC calculated mismatched; SUCCESS: CRC matched
*/
uint16_t pack_VerifyRxCRC(uint8_t *rxedBuffer)
{
	clear_crc_16();
	crc16Full(*rxedBuffer++);//sender		
	crc16Full(*rxedBuffer++);//index
	crc16Full(*rxedBuffer++);	//index
	crc16Full(*rxedBuffer++);	//ack/nack
	uint16_t calculatedCRC=crc_16;
	uint16_t receivedCRC=(*rxedBuffer++)<<8;
	receivedCRC|=*rxedBuffer++;
	if(calculatedCRC!=receivedCRC)
	{
		return CRC_MISMATCH;
	}
	else
	{
		return SUCCESS;
	}
}
/**
 * @}
 */
