/* This source file is part of the Procept Operating System */
 
/*This file is prepared for Doxygen automatic documentation generation.*/
 
/*! @file *********************************************************************
 *
 * @brief The implementation of the timer module.
 *
 * - Compiler:           uVision5 for nRF51
 * - Supported devices:  nRF51822xxAA
 * - AppNote:
 *
 * @author               tchinnadurai
 ******************************************************************************/
 
/* Copyright (c) 2011 Procept IP. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted unless the following conditions are met.
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of Procept may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with a Procept
 * Product with permission of the Procept Board of Directors.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCEPT "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 **/

#include "timer.h"
#include "nrf_drv_timer.h"
#include "app_timer.h"


#define TIMER_PRESCALERS         						(4U)//1uSec tick               // Prescaler setting for timer.
#define TIMER_PRESCALERS_PULSE_FREQ         (8U)//16uSec tick               // Prescaler setting for timer.

#define BSTR_OFF_TIME													1
#define BSTR_ON_TIME													30
#define NPWM_ON_TIME													260
#define PPWM_OFF_TIME													260
#define PPWM_ON_TIME													260

static volatile uint8_t delayCompleted 							=0;
static volatile uint8_t connParamsDelayCompleted		=0;
static volatile uint8_t pwrButtonPressTimerCompleted=0;
static volatile uint8_t startDelay 									=0;
static volatile uint8_t startConnParamsDelay				=0;
static volatile uint8_t startPwrButtonPressTimer    =0;

static volatile uint32_t timer_count 								=0;
static volatile uint32_t uart_transmit_counter			=0;
static volatile uint32_t led_timer_count						=0;
static volatile uint32_t RF_remote_radio_on_off_timer_count=0;
static volatile uint32_t rf_remote_radio_off				=0;
static volatile uint32_t number_of_50msec						=0;

static volatile uint32_t rtc1_count									=0;
static volatile uint32_t timer_delay 								=0;
static volatile uint32_t conn_params_timer_delay		=0;
static volatile uint32_t pwr_button_press_timer     =0;
static volatile uint32_t timer_delay_count 					=0;
static volatile uint32_t timer_conn_params_delay_count=0;
static volatile uint32_t timer_pwr_button_press_count=0;
static volatile uint32_t do_error_blink = 0;

static volatile uint32_t remoteRadioOffPeriod 			= REMOTE_RADIO_OFF_PERIOD/RTC1_COUNT_RF_REMOTE_CONTROL;
static volatile uint32_t remoteRadioOnPeriod 				= REMOTE_RADIO_ON_PERIOD/RTC1_COUNT_RF_REMOTE_CONTROL;

static volatile uint32_t pulseFreqTime							=0;
static volatile uint32_t NouputPeriod								=0;
static volatile uint32_t NumberOfSecondsSinceLastVbatTx = 0;

static uint8_t timer_getDelayStarted(void);
static uint8_t timer_getConnParamsDelayStarted(void);
static void wdt_event_handler(void);
static void error_setDoErrorLedBlink(void);
/************************************************
 * 				PUBLIC ROUTINES					*
 ************************************************/
 /**
 * @brief Enables the LF clock
 *
 * @param[in] 					None.
 * @returns             None.
 */
void Enable_LowFrequencyClock(void)
{
	//setup the 32kHZ Clock (LFCLK)
	NRF_CLOCK->LFCLKSRC            = (CLOCK_LFCLKSRC_SRC_Xtal << CLOCK_LFCLKSRC_SRC_Pos);
	NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
	NRF_CLOCK->TASKS_LFCLKSTART = 1;
	//wait for the clock to be running. 
	while((NRF_CLOCK->LFCLKSTAT & CLOCK_LFCLKSTAT_STATE_Msk) == 0){};
}

 /**
 * @brief Sets the rtc1 count for LF clock
 *
* @param[in] 					input_rtc1_count: The input rtc1 count
 * @returns             None.
 */
void timer_setRTC1Count(uint32_t input_rtc1_count)
{
	rtc1_count = input_rtc1_count;
}

/**
 * @brief Fetches the rtc1 count for LF clock
 *
* @param[in] 					None.
* @returns             rtc1_count: The RTC1 count for LF clock.
 */
uint32_t timer_getRTC1Count(void)
{
	return rtc1_count;
}
/*!
*	@brief 	Function for initializing the RTC1 counter.
*	@param[in] None
*	@return None.
*/
void timer_rtc1Init(void)
{
	NRF_RTC1->POWER=1;
	NRF_RTC1->PRESCALER = COUNTER_PRESCALER;
	//If COUNTER_PRESCALER = 0, then, Counter resolution = 30.517uSec.
	//Explanation: fRTC = frequency of RTC timer counter(the
	//RTC1_IRQHandler is fired at the frequency of fRTC).
	//fRTC = 32768/(PRESCALAR + 1) //(From Page:95 of nRF51
	//Series Reference Manual v1.1).
	//fRTC = 32768/(0+1) = 32768 Hz.
	//Get RTC1 interrupt every 1/32768 seconds
	//or, get RTC1 interrupt every 30.5175 u seconds
	//To get interrupt once every msec, cc counter should be
	//30.5175usec ===  1 tick
	//1msec or 1000usec === 1000/30.517 = 32.768 ticks ~= 33 ticks
	NRF_RTC1->CC[0]     = rtc1_count*33;
	NVIC_SetPriority(RTC1_IRQn, RTC1_IRQ_PRI);
}
/*!
*	@brief 	Function for starting the RTC1 timer.
*	@param[in] None.
*	@return None.
*/
void timer_rtc1Start(void)
{
		NRF_RTC1->POWER=1;	
    NRF_RTC1->EVTENSET = RTC_EVTEN_COMPARE0_Msk;
    NRF_RTC1->INTENSET = RTC_INTENSET_COMPARE0_Msk;

    NVIC_ClearPendingIRQ(RTC1_IRQn);
    NVIC_EnableIRQ(RTC1_IRQn);

    NRF_RTC1->TASKS_START = 1;
    nrf_delay_ms(100);
}

/*!
*	@brief 	Function for stopping the RTC1 timer.
*	@param[in] None.
*	@return None.
*/
void timer_rtc1Stop(void)
{
	NVIC_ClearPendingIRQ(RTC1_IRQn);
	NVIC_DisableIRQ(RTC1_IRQn);
	NRF_RTC1->TASKS_STOP = 1;
	NRF_RTC1->POWER=0;
	nrf_delay_ms(100);
}
/*!
*	@brief 	Function for assigning the delay count.
*	@param[in] None.
*	@return None.
*/
void timer_setDelay_ms(uint32_t count)
{
	timer_delay_count = count;
}

/*!
*	@brief 	Function for assigning the connection parameter delay count.
*	@param[in] None.
*	@return None.
*/
void timer_setConnParamsDelay_ms(uint32_t count)
{
	timer_conn_params_delay_count = count;
}

/*!
*	@brief 	Function for assigning the power button press timer count.
*	@param[in] None.
*	@return None.
*/
void timer_setPwrButtonPressCount(uint32_t count)
{
	timer_pwr_button_press_count = count;
}
/*!
*	@brief 	Function for starting the delay timer
*	@param[in] None.
*	@return None.
*/
void timer_startDelay(void)
{
	startDelay = 1;
}

/*!
*	@brief 	Function for starting the connection parameters delay timer
*	@param[in] None.
*	@return None.
*/
void timer_startConnParamsDelay(void)
{
	startConnParamsDelay = 1;
}

/*!
*	@brief 	Function for starting the power button press timer
*	@param[in] None.
*	@return None.
*/
void timer_startPwrButtonPressTimer(void)
{
	startPwrButtonPressTimer = 1;
}
/*!
*	@brief 	Function for stopping the delay timer
*	@param[in] None.
*	@return None.
*/
void timer_stopDelay(void)
{
	startDelay = 0;
	timer_delay = 0;
}

/*!
*	@brief 	Function for stopping the connection parameters delay timer
*	@param[in] None.
*	@return None.
*/
void timer_stopConnParamsDelay(void)
{
	startConnParamsDelay = 0;
	conn_params_timer_delay = 0;
}

/*!
*	@brief 	Function for stopping the power button press timer
*	@param[in] None.
*	@return None.
*/
void timer_stopPwrButtonPressTimer(void)
{
	startPwrButtonPressTimer = 0;
	pwr_button_press_timer = 0;
}
/*!
*	@brief 	Function for fetching delay readings
*	@param[in] None
*	@return delayCompleted
*/
uint8_t timer_getDelayCompleted (void)
{
	return delayCompleted;
}

/*!
*	@brief 	Function for fetching connection parameter delay readings
*	@param[in] None
*	@return delayCompleted
*/
uint8_t timer_getConnParamsDelayCompleted (void)
{
	return connParamsDelayCompleted;
}

/*!
*	@brief 	Function for fetching the power button press timer value
*	@param[in] None
*	@return pwrButtonPressTimerCompleted : is 1 if timeout has happened
*					once power button was pressed.
*/
uint8_t timer_getPwrButtonPressTimerCompleted (void)
{
	return pwrButtonPressTimerCompleted;
}
/*!
*	@brief 	Function for setting delay readings
*	@param[in] None
*	@return None
*/
void timer_setDelayCompleted (void)
{
	delayCompleted = 1;
}

void timer_setConnParamsDelayCompleted (void)
{
	connParamsDelayCompleted = 1;
}
void timer_setPwrButtonPressTimerCompleted(void)
{
	pwrButtonPressTimerCompleted = 1;
}


/*!
*	@brief 	Function for clearing delay readings
*	@param[in] None
*	@return None
*/
void timer_clearDelayCompleted (void)
{
	delayCompleted = 0;
}

/*!
*	@brief 	Function for clearing connection parameter delay variable
*	@param[in] None
*	@return None
*/
void timer_clearConnParamsDelayCompleted (void)
{
	connParamsDelayCompleted = 0;
}


/*!
*	@brief 	Function for clearing power button delay variable
*	@param[in] None
*	@return None
*/
void timer_clearPwrButtonPressTimerCompleted (void)
{
	pwrButtonPressTimerCompleted = 0;
}

/*!
*	@brief 	Function to fetch the RTC1 count.
*	@param[in] 		None
* @retval     	timer_count : RTC1 count
*/
uint32_t timer_getRtc1Count(void)
{
	return timer_count;
}

/*!
*	@brief 	Function to set the P output pulse period width in usec.
*	@param[in] 		periodInMicroSeconds: Input P ouput pulse period in usec.
* @retval     	None
*/
void timer_setPulseFreqTime(uint32_t microSecondsInput)
{
	pulseFreqTime=microSecondsInput;
}

/*!
*	@brief 	Function to set the N output pulse period width in usec.
*	@param[in] 		periodInMicroSeconds: Input N ouput pulse period in usec.
* @retval     	None
*/
void timer_setNouputPeriod(uint32_t periodInMicroSeconds)
{
	NouputPeriod=periodInMicroSeconds;
}

/*!
*	@brief 	Function to pulse frequency timer for N and P PWM pulses.
*	@param[in] 		None
* @retval     	None
*/
void timer_initPulseFrequency(void)
{

    NRF_TIMER1->MODE      = TIMER_MODE_MODE_Timer;
    NRF_TIMER1->BITMODE   = TIMER_BITMODE_BITMODE_16Bit << TIMER_BITMODE_BITMODE_Pos;
    NRF_TIMER1->PRESCALER = TIMER_PRESCALERS_PULSE_FREQ;

    // Clears the timer, sets it to 0.
    NRF_TIMER1->TASKS_CLEAR = 1;
		
    // Load the initial values to TIMER2 CC registers.
	
    NRF_TIMER1->CC[0] = 1;
		//need 8.333msec timer; so, 8333usec - 16(off tick) =  - (260*3.5)=.../16 = 463
	//130usec  is the time taken to set up the timers and ppi enables
		double tmrcount = (pulseFreqTime-((3*NouputPeriod)+130))/16;
		NRF_TIMER1->CC[1] = (uint16_t)tmrcount;

    // CC2 will be set on the first CC1 interrupt.
		NRF_TIMER1->SHORTS=(
												 NRF_TIMER_SHORT_COMPARE1_STOP_MASK  |
												 NRF_TIMER_SHORT_COMPARE1_CLEAR_MASK);
    // Interrupt setup.
    NRF_TIMER1->INTENSET =  (TIMER_INTENSET_COMPARE0_Enabled << TIMER_INTENSET_COMPARE0_Pos)
														| (TIMER_INTENSET_COMPARE1_Enabled << TIMER_INTENSET_COMPARE1_Pos);
}

/*!
*	@brief 	Function to initialise the timer registers for Booster PWM pulse generation.
*	@param[in] 		None
* @retval     	None
*/
void timer_initBoosterPwm(void)
{

    NRF_TIMER2->MODE      = TIMER_MODE_MODE_Timer;
    NRF_TIMER2->BITMODE   = TIMER_BITMODE_BITMODE_16Bit << TIMER_BITMODE_BITMODE_Pos;
    NRF_TIMER2->PRESCALER = TIMER_PRESCALERS;

    // Clears the timer, sets it to 0.
    NRF_TIMER2->TASKS_CLEAR = 1;
		
    // Load the initial values to TIMER2 CC registers.
	
    NRF_TIMER2->CC[0] = BSTR_OFF_TIME;
    NRF_TIMER2->CC[1] = BSTR_ON_TIME+BSTR_OFF_TIME;

    // CC2 will be set on the first CC1 interrupt.
		NRF_TIMER2->SHORTS=(
												 NRF_TIMER_SHORT_COMPARE1_STOP_MASK  |
												 NRF_TIMER_SHORT_COMPARE1_CLEAR_MASK |
												 NRF_TIMER_SHORT_COMPARE2_CLEAR_MASK);
    // Interrupt setup.
    NRF_TIMER2->INTENSET =  (TIMER_INTENSET_COMPARE0_Enabled << TIMER_INTENSET_COMPARE0_Pos)
														| (TIMER_INTENSET_COMPARE1_Enabled << TIMER_INTENSET_COMPARE1_Pos);
}

/*!
*	@brief 	Function to get ready to start the timer for Booster PWM pulse generation.
*	@param[in] 		None
* @retval     	None
*/
void setup_timerBoosterPwm(void)
{
    NRF_POWER->TASKS_CONSTLAT = 1;

    // Enable interrupt on Timer 2.
    NVIC_EnableIRQ(TIMER2_IRQn);
    __enable_irq();

    // Workaround for PAN-73: Use of an EVENT from any TIMER module to trigger a TASK in GPIOTE or 
    // RTC using the PPI could fail under certain conditions.
    *(uint32_t *)0x4000AC0C = 1;    
}
/*!
*	@brief 	Function to start the timer for Booster PWM pulse generation.
*	@param[in] 		None
* @retval     	None
*/
void start_timerBoosterPwm(void)
{
    NRF_POWER->TASKS_CONSTLAT = 1;

    // Enable interrupt on Timer 2.
    NVIC_EnableIRQ(TIMER2_IRQn);
    __enable_irq();

    // Workaround for PAN-73: Use of an EVENT from any TIMER module to trigger a TASK in GPIOTE or 
    // RTC using the PPI could fail under certain conditions.
    *(uint32_t *)0x4000AC0C = 1;    
    // Start the timer.
		NRF_TIMER2->TASKS_START = 1;	
}

/*!
*	@brief 	Function to set up the pulse frequency for timer that is used
*					in Booster PWM generation.
*	@param[in] 		None
* @retval     	None
*/
void setup_timerPulseFreq(void)
{
    NRF_POWER->TASKS_CONSTLAT = 1;

    // Enable interrupt on Timer 2.
    NVIC_EnableIRQ(TIMER1_IRQn);
    __enable_irq();

    // Workaround for PAN-73: Use of an EVENT from any TIMER module to trigger a TASK in GPIOTE or 
    // RTC using the PPI could fail under certain conditions.
    *(uint32_t *)0x4000AC0C = 1;    
}

/*!
*	@brief 	Function to initialise the timer needed for N and P PWM generation
*	@param[in] 		None
* @retval     	None
*/
void timer_initNPwm(void)
{

    NRF_TIMER2->MODE      = TIMER_MODE_MODE_Timer;
    NRF_TIMER2->BITMODE   = TIMER_BITMODE_BITMODE_16Bit << TIMER_BITMODE_BITMODE_Pos;
    NRF_TIMER2->PRESCALER = TIMER_PRESCALERS;

    // Clears the timer, sets it to 0.
    NRF_TIMER2->TASKS_CLEAR = 1;
		
    // Load the initial values to TIMER2 CC registers.
	
    NRF_TIMER2->CC[0] = NouputPeriod;
    NRF_TIMER2->CC[1] = NouputPeriod+NouputPeriod;
		NRF_TIMER2->CC[2] = NouputPeriod+NouputPeriod+NouputPeriod;

    // CC2 will be set on the first CC1 interrupt.
		NRF_TIMER2->SHORTS=(
												 NRF_TIMER_SHORT_COMPARE2_STOP_MASK  |
												 NRF_TIMER_SHORT_COMPARE2_CLEAR_MASK);
    // Interrupt setup.
    NRF_TIMER2->INTENSET =  (TIMER_INTENSET_COMPARE0_Enabled << TIMER_INTENSET_COMPARE0_Pos)
														| (TIMER_INTENSET_COMPARE1_Enabled << TIMER_INTENSET_COMPARE1_Pos)
														| (TIMER_INTENSET_COMPARE2_Enabled << TIMER_INTENSET_COMPARE2_Pos);
}
 
 
/*!
*	@brief 	Function to extend the time for which Radio needs to be ON.
*					This is called whenever a response needed to be sent back to the
*					remote control module.
*	@param[in] 		None
* @retval     	None
*/
void timer_extendRadioOnPeriod(void)
{
	remoteRadioOnPeriod+=REMOTE_RADIO_EXTENDED_ON_PERIOD_TO_SEND_RESPONSE;
}


/*!
*	@brief 	Function for clearing the number of seconds
*					since last VBAT.
*	@param[in] None
*	@return None
*/
void timer_clearNumberOfSecondsSinceLastVbatTx(void)
{
	NumberOfSecondsSinceLastVbatTx = 0;
}
uint8_t timer_getPwrButtonPressTimerStarted(void)
{
	return startPwrButtonPressTimer;
}
uint32_t timer_getUartTransmitCounter(void)
{
	return uart_transmit_counter;
}
void timer_clearUartTransmitCounter(void)
{
	uart_transmit_counter=0;
}
/*!
*	@brief 	RTC1 Interrupt Service Routine
*	@param[in] None
*	@return None
*/
void RTC1_IRQHandler(void)
{
	if ((NRF_RTC1->EVENTS_COMPARE[0] != 0))
	{
		NRF_RTC1->EVENTS_COMPARE[0] = 0;
		NRF_RTC1->TASKS_CLEAR = 1;
		timer_count++;
		uart_transmit_counter++;
	/*	led_timer_count++;
		
		if(timer_getDelayStarted())
		{
			timer_delay++;
			if(timer_delay>=timer_delay_count/rtc1_count)
			{
				timer_setDelayCompleted();
				timer_stopDelay();
			}
		}
		if(timer_getConnParamsDelayStarted())
		{
			conn_params_timer_delay++;
			if(conn_params_timer_delay>=timer_conn_params_delay_count/rtc1_count)
			{
				timer_setConnParamsDelayCompleted();
				timer_stopConnParamsDelay();
			}			
		}
		if(timer_getPwrButtonPressTimerStarted())
		{
			pwr_button_press_timer++;
			if(pwr_button_press_timer>=timer_pwr_button_press_count/rtc1_count)
			{
				timer_setPwrButtonPressTimerCompleted();
				timer_stopPwrButtonPressTimer();
			}
		}*/
	}
	// Clear all events (also unexpected ones)
	NRF_RTC1->EVENTS_COMPARE[1] = 0;
	NRF_RTC1->EVENTS_COMPARE[2] = 0;
	NRF_RTC1->EVENTS_COMPARE[3] = 0;
	NRF_RTC1->EVENTS_TICK       = 0;
	NRF_RTC1->EVENTS_OVRFLW     = 0;
	
    // Clear all events (also unexpected ones)
	NRF_RTC1->EVENTS_COMPARE[0] = 0;
	NRF_RTC1->EVENTS_TICK       = 0;
	NRF_RTC1->EVENTS_OVRFLW     = 0;

    // Check for expired timers
    timer_timeouts_check();
}

/************************************************
 * 				PRIVATE ROUTINES					*
 ************************************************/

/*!
*	@brief 	Function for fetching delay timer for button has started
*	@param[in] None
*	@return startDelay
*/
static uint8_t timer_getDelayStarted(void)
{
	return startDelay;
}

/*!
*	@brief 	Function for resetting micro when wdt event fires.
*	@param[in] None
*	@return None
*/
static void wdt_event_handler(void)
{ 
	NVIC_SystemReset();
}
static uint8_t timer_getConnParamsDelayStarted(void)
{
	return startConnParamsDelay;
}
static void error_setDoErrorLedBlink(void)
{
	do_error_blink = 1;
}

/**
 * @}
 */
