/* This source file is part of the Procept Operating System */
 
/*This file is prepared for Doxygen automatic documentation generation.*/
 
/*! @file *********************************************************************
 *
 * @brief The header file for timer.c
 *
 * - Compiler:           uVision5 for nRF51
 * - Supported devices:  nRF51822xxAA
 * - AppNote:
 *
 * @author               tchinnadurai
 ******************************************************************************/
 
/* Copyright (c) 2011 Procept IP. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted unless the following conditions are met.
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of Procept may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with a Procept
 * Product with permission of the Procept Board of Directors.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCEPT "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 **/


#ifndef TIMER_H__
#define TIMER_H__

#include <stdbool.h>
#include <stdint.h>
#include "nordic_common.h"
#include "nrf.h"
#include "nrf_delay.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"

#define RTC1_IRQ_PRI            APP_IRQ_PRIORITY_LOW                        /**< Priority of the RTC1 interrupt (used for checking for timeouts and executing timeout handlers). */

//We have different RTC interrupt ticks for Bluetooth mode and RF remote
//control modes because, in RF remote control mode, minimum RTC tick 
//needs to be 5 msec(because the RF would be On for 5 msec and off for 200msec
//to save power).
//In Bluetooth mode, there is no restriction for such a small RTC tick.
//So, the minimum RTC tick needed would be 50 msec(for led on time).
//To be on the safe side, instead of 50msec ticks, 25 msec ticks are used for 
//Bluetooth mode.
#define RTC1_COUNT_BLEUTOOTH							25//milliseconds; 1 milliseconds because, Radio On time for Remote control mode is 5 msec
#define RTC1_COUNT_RF_REMOTE_CONTROL			1//5//milliseconds; 1 milliseconds because, Radio On time for Remote control mode is 5 msec

#define RTC1_COUNT_SLOW		  			500//milliseconds
#define LFCLK_FREQUENCY           (32768UL)                               /**< LFCLK frequency in Hertz, constant. */
#define RTC_FREQUENCY             (32768UL)//(1100UL)                                 /**< Required RTC working clock RTC_FREQUENCY Hertz. Changable. */

#define COUNTER_PRESCALER         ((LFCLK_FREQUENCY / RTC_FREQUENCY) - 1)   // f = LFCLK/(prescaler + 1) 
#define TIME_BIN									50//in milliseconds. This is 50msec because minimum time Led is on is 50msec (example: active, standby states)
#define TIMEOUT_BLE_NOTIFY	5000//5 seconds in milliseconds
#define PAIRING_TIMEOUT	20000//20 seconds in milliseconds
#define PAUSE_TIMEOUT	1800000//30 minutes in milliseconds
#define STANDBY_TIMEOUT	7200000//2 hours in milliseconds
#define WATCHDOG_TIMEOUT	5000//5 seconds
#define BUTTON_RELEASE_TIMEOUT	6000//6 seconds in milliseconds
#define REMOTE_RADIO_OFF_PERIOD	200//in milliseconds
#define REMOTE_RADIO_ON_PERIOD	5//in milliseconds
#define REMOTE_RADIO_EXTENDED_ON_PERIOD_TO_SEND_RESPONSE	50//in milliseconds
#define PRESCALAR_TIMER0_PRODUCTION_FOR_16MHZ	10
#define VBAT_UPDATE_TIME_STANDBY_STATE 1//in seconds
#define TIME_TO_TURN_YELLOW_ON	5//in milliseconds
#define TIME_TO_TURN_YELLOW_OFF	10//in milliseconds
#define TIME_TO_TURN_GREEN_ON	15//in milliseconds
#define TIME_TO_TURN_GREEN_OFF	20//in milliseconds
#define NUMBER_OF_CYCLES_ERROR_BLINK	2
#define STANDBY_ON_TIME	1//milliseconds
#define STANDBY_OFF_TIME 2//milliseconds
#define STANDBY_LED_CHANGEOVER_TIME	20//milliseconds
#define ACTIVE_ON_TIME	1//milliseconds
#define ACTIVE_OFF_TIME 2//milliseconds
#define ACTIVE_LED_CHANGEOVER_TIME	10//milliseconds
#define MAX_INTENSITY_LED_ON_FIRST	1//milliseconds
#define MAX_INTENSITY_LED_OFF_FIRST	2//milliseconds
#define MAX_INTENSITY_LED_ON_SECOND	7//milliseconds
#define MAX_INTENSITY_LED_OFF_SECOND	8//milliseconds
#define PAUSE_ON_TIME	1//in times of 50msec milliseconds
#define PAUSE_ON_TIME_2	7//in times of 50msec milliseconds
#define PAUSE_OFF_TIME 2//in times of 50msec milliseconds
#define PAUSE_OFF_TIME_2 8//in times of 50msec milliseconds
#define PAUSE_LED_CHANGEOVER_TIME	20//milliseconds

/**
 * @brief Enables the LF clock
 *
 * @param[in] 					None.
 * @returns             None.
 */
void Enable_LowFrequencyClock(void);

 /**
 * @brief Sets the rtc1 count for LF clock
 *
* @param[in] 					input_rtc1_count: The input rtc1 count
 * @returns             None.
 */
void timer_setRTC1Count(uint32_t input_rtc1_count);

/*!
*	@brief 	Function for initializing the RTC1 counter.
*	@param[in] None
*	@return None.
*/
void timer_rtc1Init(void);

/*!
*	@brief 	Function for starting the RTC1 timer.
*	@param[in] None.
*	@return None.
*/
void timer_rtc1Start(void);

/*!
*	@brief 	Function for assigning the delay count.
*	@param[in] None.
*	@return None.
*/
void timer_setDelay_ms(uint32_t count);

/*!
*	@brief 	Function for starting the delay timer
*	@param[in] None.
*	@return None.
*/
void timer_startDelay(void);

/*!
*	@brief 	Function for fetching delay between successive ADC readings
*	@param[in] None
*	@return delayCompleted
*/
uint8_t timer_getDelayCompleted (void);

/*!
*	@brief 	Function to initialise the timer registers for Booster PWM pulse generation.
*	@param[in] 		None
* @retval     	None
*/
void timer_initBoosterPwm(void);

/*!
*	@brief 	Function to get ready to start the timer for Booster PWM pulse generation.
*	@param[in] 		None
* @retval     	None
*/
void setup_timerBoosterPwm(void);

/*!
*	@brief 	Function to start the timer for Booster PWM pulse generation.
*	@param[in] 		None
* @retval     	None
*/
void start_timerBoosterPwm(void);

/*!
*	@brief 	Function to pulse frequency timer for N and P PWM pulses.
*	@param[in] 		None
* @retval     	None
*/
void timer_initPulseFrequency(void);

/*!
*	@brief 	Function to set up the pulse frequency for timer that is used
*					in Booster PWM generation.
*	@param[in] 		None
* @retval     	None
*/
void setup_timerPulseFreq(void);

/*!
*	@brief 	Function to initialise the timer needed for N and P PWM generation
*	@param[in] 		None
* @retval     	None
*/
void timer_initNPwm(void);

/*!
*	@brief 	Function to set the P output pulse period width in usec.
*	@param[in] 		periodInMicroSeconds: Input P ouput pulse period in usec.
* @retval     	None
*/
void timer_setPulseFreqTime(uint32_t microSecondsInput);

/*!
*	@brief 	Function to set the N output pulse period width in usec.
*	@param[in] 		periodInMicroSeconds: Input N ouput pulse period in usec.
* @retval     	None
*/
void timer_setNouputPeriod(uint32_t periodInMicroSeconds);

/*!
*	@brief 	Function to fetch the RTC1 count.
*	@param[in] 		None
* @retval     	timer_count : RTC1 count
*/
uint32_t timer_getRtc1Count(void);

/*!
*	@brief 	Function to flash Led during RF to BT change state.
*	@param[in] 		None
* @retval     	None
*/
void timer_ledFlashHandlerBTChangeState(void);

/*!
*	@brief 	Function to flash Led during BT to RF change state.
*	@param[in] 		None
* @retval     	None
*/
void timer_ledFlashHandlerRFChangeState(void);

/*!
*	@brief 	Function to flash the LED according to which state the Tens Device is.
*	@param[in] 		None
* @retval     	None
*/
void timer_ledFlashHandler(void);

/*!
*	@brief 	Function for stopping the delay timer
*	@param[in] None.
*	@return None.
*/
void timer_stopDelay(void);

/*!
*	@brief 	Function for clearing delay
*	@param[in] None
*	@return None
*/
void timer_clearDelayCompleted(void);

/*!
*	@brief 	Function to extend the time for which Radio needs to be ON.
*					This is called whenever a response needed to be sent back to the
*					remote control module.
*	@param[in] 		None
* @retval     	None
*/
void timer_extendRadioOnPeriod(void);

/*!
*	@brief 	Function for stopping the RTC1 timer.
*	@param[in] None.
*	@return None.
*/
void timer_rtc1Stop(void);

/**
*	@brief 	Function for clearing the number of seconds
*					since last VBAT.
*	@param[in] None
*	@return None
*/
void timer_clearNumberOfSecondsSinceLastVbatTx(void);

/**
 * @brief Fetches the rtc1 count for LF clock
 *
* @param[in] 					None.
* @returns             rtc1_count: The RTC1 count for LF clock.
 */
uint32_t timer_getRTC1Count(void);

/*!
*	@brief 	Function for intialising and setting the watchdog timer timeout.
*					
*	@param[in] watchdogTimeoutValue : watch dog timeout value
*	@return None
*/
void timer_watchDogTimerInit(uint32_t watchdogTimeoutValue);

/*!
*	@brief 	Function for feeding the watch dog timer.
*	@param[in] None
*	@return None
*/
void timer_watchDogFeed(void);

/*!
*	@brief 	Function for stopping the connection parameters delay timer
*	@param[in] None.
*	@return None.
*/
void timer_stopConnParamsDelay(void);

/*!
*	@brief 	Function for clearing connection parameter delay variable
*	@param[in] None
*	@return None
*/
void timer_clearConnParamsDelayCompleted (void);

/*!
*	@brief 	Function for fetching connection parameter delay readings
*	@param[in] None
*	@return delayCompleted
*/
uint8_t timer_getConnParamsDelayCompleted (void);

/*!
*	@brief 	Function for assigning the connection parameter delay count.
*	@param[in] None.
*	@return None.
*/
void timer_setConnParamsDelay_ms(uint32_t count);

/*!
*	@brief 	Function for starting the connection parameters delay timer
*	@param[in] None.
*	@return None.
*/
void timer_startConnParamsDelay(void);

/*!
*	@brief 	Function for assigning the power button press timer count.
*	@param[in] None.
*	@return None.
*/
void timer_setPwrButtonPressCount(uint32_t count);

/*!
*	@brief 	Function for starting the power button press timer
*	@param[in] None.
*	@return None.
*/
void timer_startPwrButtonPressTimer(void);

/*!
*	@brief 	Function for fetching the power button press timer value
*	@param[in] None
*	@return pwrButtonPressTimerCompleted : is 1 if timeout has happened
*					once power button was pressed.
*/
uint8_t timer_getPwrButtonPressTimerCompleted (void);

/*!
*	@brief 	Function to check if power button has been released.
*	@param[in] 		None
* @retval     	None
*/
void timer_checkPowerButtonReleaseCondition(void);

/*!
*	@brief 	Function for stopping the power button press timer
*	@param[in] None.
*	@return None.
*/
void timer_stopPwrButtonPressTimer(void);

/*!
*	@brief 	Function for clearing power button delay variable
*	@param[in] None
*	@return None
*/
void timer_clearPwrButtonPressTimerCompleted (void);

/*!
*	@brief 	Function for fetching the led to flash.
*	@param[in] None
*	@return None
*/
uint32_t timer_getLedToFlash(void);
uint8_t timer_getPwrButtonPressTimerStarted(void);
uint32_t timer_getUartTransmitCounter(void);
void timer_clearUartTransmitCounter(void);
#endif // TIMER_H__
/** @} */
